/***********************************************************************
*
*  Program: USERS.C  List Netware users
*  Author: Paul Steele
*  Date-written: 05-Dec-89
*
*  This program will list all users on the current file server, along
*  with the user's last login date and time, and expiry date.  The
*  program can be easily expanded/changed depending on what the user
*  wants.  See the UserInfoRecord struct below.  I had to obtain the
*  infomation in this structure by trial and error since there did not
*  seem to be any description of the property's contents in the Netware
*  API manuals.  There are some fields I couldn't find a use for.
*
***********************************************************************/
#include <nit.h>
#include <niterror.h>

typedef unsigned char byte;

typedef struct {
  byte userExpiryDate[3];  /* date account expires */
  byte disabled;           /* account is disabled? */
  byte pwExpiryDate[3];    /* password expiry date */
  byte graceLoginsRemaining;  /* number of grace logins remaining */
  int passwordDays;  /* number of days between forced password changes */
  byte graceLoginsAllowed;  /* -1 means do not limit grace logins */
  byte minPasswordLength;   /* minimum allowed password length */
  byte extra1;
  byte maxConnections;      /* maximum number of allowed connections */
  byte timeRestrictions[42];
  byte lastLoginDate[3];
  byte lastLoginTime[3];
  byte extra2[2];
  long maxQuota;            /* user's maximum allowed disk quota */
  byte extra3[60];
} UserInfoRecord;

void printDT(byte *dt, char sep)
{
  printf("%02d%c%02d%c%02d", dt[0], sep, dt[1], sep, dt[2]);
}

main()
{
  int objectType,len,i;
  long objectID = -1;
  char username[48], propval[128], property[16], properties, more;
  UserInfoRecord info;

  printf("%-10s  %-30s %-17s  %-8s\n", "Username", "Fullname",
	"Last Login Date", "Expiry");
  for (i=0; i<70; i++) printf("-");
  printf("\n");
  while (!ScanBinderyObject("*", OT_USER, &objectID, username,
		 NULL, NULL, NULL, NULL)) {
	if (ReadPropertyValue(username, OT_USER, "IDENTIFICATION", 1,
		propval, NULL, NULL) != 0)
	  printf("%-10s  %-30s ", username, "(No Name)");
	else
	  printf("%-10s  %-30s ", username, propval);
	ReadPropertyValue(username, OT_USER, "LOGIN_CONTROL", 1,
		(char *)&info, &more, NULL);
	if (info.lastLoginDate[0] == 0)
	  printf("%-17s","(No logins)");
	else {
	  printDT(info.lastLoginDate,'-');
	  printf(" ");
	  printDT(info.lastLoginTime,':');
	}
	printf("  ");
	if (info.userExpiryDate[0] == 0)
	  printf("%-8s","(None)");
	else
	  printDT(info.userExpiryDate,'-');
	printf("\n");
  }
}